/*
 * This work is licensed under the terms of the GNU LGPL, version 2.1 or later.
 * See the COPYING file in the top-level directory.
 */
#ifndef LIBCACARD_H
#define LIBCACARD_H

#include <glib.h>

G_BEGIN_DECLS

#define __LIBCACARD_H_INSIDE__

#include "cac.h"
#include "card_7816.h"
#include "card_7816t.h"
#include "eventt.h"
#include "vcard_emul.h"
#include "vcard_emul_type.h"
#include "vcard.h"
#include "vcardt.h"
#include "vevent.h"
#include "vreader.h"
#include "vreadert.h"
#include "vscard_common.h"

G_END_DECLS

#endif
